/*
 * (c) 2023, SWD Embedded Systems Limited, http://www.kpda.ru
 */


#ifndef __SUN8I_H__
#define __SUN8I_H__

#define BIT(bit)				(UINT32_C(1) << bit)
#define GENMASK32(a, b)				((UINT32_MAX >> (32 - (b) - 1)) & (UINT32_MAX << (a)))

// Syscon address space
#define AW_H3_SYS_CTRL_BASE			0x01c00000
#define AW_H3_SYS_CTRL_SIZE			0x1000

// EMAC Clock Register, Syscon
#define EMAC_CLK_REG				0x0030
#define H3_EPHY_DEFAULT_VALUE			0x58000
#define H3_EPHY_DEFAULT_MASK			GENMASK32(16, 31)
#define H3_EPHY_ADDR_SHIFT			20
#define REG_PHY_ADDR_MASK			GENMASK32(0, 4)
#define SC_EPIT					BIT(2) // 1: RGMII, 0: MII
#define SC_RMII_EN				BIT(13)
#define H3_EPHY_SELECT				BIT(15) // 1: internal PHY, 0: external PHY
#define H3_EPHY_SHUTDOWN			BIT(16)	// 1: shutdown, 0: power up
#define H3_EPHY_LED_POL				BIT(17)	// 1: active low, 0: active high
#define H3_EPHY_CLK_SEL				BIT(18) // 1: 24MHz, 0: 25MHz
#define SC_ETCS_MASK				0x3

// CCU address space
#define AW_H3_CCU_BASE				0x01c20000
#define AW_H3_CCU_SIZE				0x400

// Clock source register, CCU
#define BUS_CLK_GATING_REG0			0x0060
#define BUS_CLK_GATING_REG0_EMAC_GATING 	BIT(17)

// Reset source register, CCU
#define BUS_SOFT_RST_REG0			0x02C0
#define BUS_SOFT_RST_REG0_EMAC_RST		BIT(17)

// EPHY clock, CCU
#define BUS_CLK_GATING_REG4			0x0070
#define BUS_CLK_GATING_REG4_EPHY_GATING		BIT(0)

// EPHY reset, CCU
#define BUS_SOFT_RST_REG2			0x02c8
#define BUS_SOFT_RST_REG2_EPHY_RST		BIT(2)

// EMAC address space
#define SUN8I_EMAC_BASE				0x01c30000
#define SUN8I_EMAC_SIZE				0x10000

// EMAC Basic Control 0 Register (Link control)
#define EMAC_CTL0				0x0000
#define EMAC_CTL0_FULL_DUPLEX			BIT(0)
#define EMAC_CTL0_LOOPBACK			BIT(1)
#define EMAC_CTL0_SPEED_MASK			(BIT(3) | BIT(2))
#define EMAC_CTL0_10				(BIT(3))
#define EMAC_CTL0_100				(BIT(3) | BIT(2))

// EMAC Basic Control 1 Register
#define EMAC_CTL1				0x0004
#define EMAC_CTL1_SOFT_RST			BIT(0) // Soft reset
#define EMAC_CTL1_BURST_SIZE(val)		((val & 0x3f) << 24) // RX/TX DMA transfer burst len

// EMAC Interrupt Status/Enable Register
#define EMAC_INT_STA				0x0008
#define EMAC_INT_EN				0x000c
#define EMAC_INT_RX_INT				BIT(8) // RX interrupt
#define EMAC_INT_TX_INT				BIT(0) // TX interrupt
#define EMAC_INT_TX_UNDERFLOW_INT_EN		BIT(4) // TX FIFO underflow error
#define EMAC_INT_RX_UNDERFLOW_INT_EN		BIT(12) // RX FIFO underflow error
#define EMAC_INT_EN_MASK			(EMAC_INT_RX_INT | EMAC_INT_TX_INT | EMAC_INT_RX_UNDERFLOW_INT_EN | EMAC_INT_TX_UNDERFLOW_INT_EN)

// EMAC Transmit Control 0 Register
#define EMAC_TX_CTL0				0x0010
#define EMAC_TX_CTL0_TX_EN			BIT(31) // Enable transmitter

// EMAC Transmit Control 1 Register
#define EMAC_TX_CTL1				0x0014
#define EMAC_TX_CTL1_TX_MD			BIT(1) // Transmission starts after a full frame located in TX DMA FIFO
#define EMAC_TX_CTL1_TX_DMA_EN			BIT(30) // Start and run TX DMA
#define EMAC_TX_CTL1_TX_DMA_START		BIT(31) // Start and run TX DMA

// EMAC Transmit DMA Descriptor List Address Register
#define EMAC_TX_DMA_DESC			0x0020

// EMAC Receive Control 0 Register
#define EMAC_RX_CTL0				0x0024
#define EMAC_RX_CTL0_RX_EN			BIT(31) // Enable receiver

// EMAC Receive Control 1 Register
#define EMAC_RX_CTL1				0x0028
#define EMAC_RX_CTL1_RX_MD			BIT(1) // RX DMA reads data from RX DMA FIFO to host memory after a complete frame has been written to RX DMA FIFO
#define EMAC_RX_CTL1_RX_DMA_EN			BIT(30) // Start and run RX DMA

// EMAC Receive DMA Descriptor List Address Register
#define EMAC_RX_DMA_DESC			0x0034

// EMAC MAC Address 0 High Register
#define EMAC_ADDR0_HIGH				0x0050

// EMAC MAC Address 0 Low Register
#define EMAC_ADDR0_LOW				0x0054

// EMAC MII Command Register
#define EMAC_MII_CMD				0x0048
#define MDIO_CMD_MII_BUSY			BIT(0)
#define MDIO_CMD_MII_WRITE			BIT(1)
#define MDIO_CMD_MII_PHY_REG_ADDR_MASK		GENMASK32(4, 8)
#define MDIO_CMD_MII_PHY_REG_ADDR_SHIFT		4
#define MDIO_CMD_MII_PHY_ADDR_MASK		GENMASK32(12, 16)
#define MDIO_CMD_MII_PHY_ADDR_SHIFT		12
#define MDIO_CMD_MDC_DIV_RATIO_M_128		(BIT(20) | BIT(21))

// EMAC MII Data Register
#define EMAC_MII_DATA				0x004c

// Descriptors macros
#define CONFIG_ETH_BUFSIZE			2048 // Note must be dma aligned
#define CONFIG_ETH_RXSIZE			2044 // Note must fit in ETH_BUFSIZE
#define SUN8I_DESC(buf, idx)			(&((sun8i_desc_t *)(buf)->area)[(idx)])

// TX descriptors
#define CONFIG_TX_DESCR_NUM			32
#define SUN8I_TX_DESC(sun8i, idx)		SUN8I_DESC(&(sun8i)->txb, (idx))
#define TX_DESC_CTL				BIT(31)
#define TX_INT_CTL				BIT(31)
#define LAST_DESC				BIT(30)
#define FIR_DESC				BIT(29)
#define TX_HEADER_ERR				BIT(16)
#define TX_LENGHT_ERR				BIT(14)
#define TX_PAYLOAD_ERR				BIT(12)
#define TX_CRS_ERR				BIT(10)
#define TX_COL_ERR_0				BIT(9)
#define TX_COL_ERR_1				BIT(8)
#define TX_COL_CNT_MASK				GENMASK32(3, 6)
#define TX_COL_CNT_SHIFT			3
#define TX_COL_CNT(status)			(((status) & TX_COL_CNT_MASK) >> TX_COL_CNT_SHIFT)
#define TX_DEFER_ERR				BIT(2)
#define TX_UNDERFLOW_ERR			BIT(1)
#define TX_DEFER				BIT(0)
#define ERROR_STATUS_MASK			(TX_HEADER_ERR | TX_LENGHT_ERR | TX_PAYLOAD_ERR | TX_CRS_ERR | TX_COL_ERR_0 | TX_DEFER_ERR | TX_UNDERFLOW_ERR)

// RX descriptors
#define CONFIG_RX_DESCR_NUM			32
#define SUN8I_RX_DESC(sun8i, idx)		SUN8I_DESC(&(sun8i)->rxb, (idx))
#define RX_DESC_CTL				BIT(31)
#define RX_LAST_DESC				BIT(8)
#define RX_DESC_FRM_SIZE_MASK			GENMASK32(16, 29)
#define RX_DESC_FRM_SIZE(status)		(((status) & RX_DESC_FRM_SIZE_MASK) >> 16)

#define CONFIG_SYS_HZ				1000
#define CONFIG_MDIO_TIMEOUT			(3 * CONFIG_SYS_HZ)
#define EMAC_SOFT_RESET_TIMEOUT_USEC		(100 * 1000)

#define SPI_INTERRUPT_START			0x20
#define SUN8I_H3_INTERRUPT			(SPI_INTERRUPT_START + 0x52)

const uint8_t etherbroadcastaddr[ETHER_ADDR_LEN] =
{ 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };

enum emac_variant
{
	H3_EMAC = 2,
};

enum
{
	PHY_INTERFACE_MODE_MII = 1,
	PHY_INTERFACE_MODE_GMII,
	PHY_INTERFACE_MODE_SGMII,
};

struct sun8i_desc
{
	uint32_t				status;
	uint32_t				size;
	uint32_t				addr;
	uint32_t				next;
} __attribute__ ((packed));

typedef struct sun8i_desc sun8i_desc_t;

struct desc_buffer
{
	void				*area;
	size_t				size;
	off_t				dma;
	int				prot_flags;
	int				map_flags;
	int				tmem_fd;

	struct mbuf			**mbufs_set;

	int				head;
	int				tail;
	int				cnt;
	int				free_cnt;
	struct
	{
		uint64_t		header;
		uint64_t		length;
		uint64_t		payload;
		uint64_t		crs;
		uint64_t		coll0;
		uint64_t		coll1;
		uint64_t		coll_cnt;
		uint64_t		defer;
		uint64_t		underflow;
	} err;
};

typedef struct desc_buffer desc_buffer_t;

struct sun8i_dev
{
	struct device				dev;
	struct ethercom				ec;
	struct ieee80211com			ic;
	struct mii_data				bsd_mii;
	int					iid;
	int					intr_cnt;
	int					intr_spurious;
	void					*sd_hook;

	uintptr_t				syscon_base;
	uintptr_t				clk_base;
	uintptr_t				emac_base;

	enum emac_variant			variant;

	int					use_internal_phy;

	uint32_t				mii_mode;
	uint32_t				adv;
	nic_config_t				cfg;
	nic_stats_t				stats;
	struct _iopkt_self			*iopkt;
	struct _iopkt_inter			inter;

	int					speed;
	int					duplex;

	desc_buffer_t				txb;
	desc_buffer_t				rxb;

	struct cache_ctrl			cachectl;
	uint32_t				irq_src;
	mdi_t					*mdi;
	struct callout				mii_callout;
	int					linkup;
	int					start_running;
};

typedef struct sun8i_dev sun8i_dev_t;

struct sun8i_attach_args
{
	uintptr_t				syscon_base;
	uintptr_t				clk_base;
	uintptr_t				emac_base;

	enum emac_variant			variant;

	int					use_internal_phy;
	int					phy_addr;

	uint32_t				mii_mode;
	nic_config_t				cfg;
	struct _iopkt_self			*iopkt;
	char					*options;
};

#endif
