/*
 * (c) 2023, SWD Embedded Systems Limited, http://www.kpda.ru
 */

/*
 * SUN8I SMP support
 */

#include "startup.h"
#include <arm/aw_h3.h>

#define BIT(nr) (1UL << (nr))

extern void smp_spin();

unsigned	startup_smp_start;

unsigned
board_smp_num_cpu()
{
	if (debug_flag) {
		kprintf("Initializing SMP for %d CPUs...\n", AW_H3_NUM_OF_CORES);
	}

	return AW_H3_NUM_OF_CORES;
}

void
board_smp_init(struct smp_entry *smp, unsigned num_cpus)
{
	smp->send_ipi = (void *)&sendipi_gic;
}

int
board_smp_start(unsigned cpu, void (*start)(void))
{	
	uint32_t reg;
	startup_smp_start = (uint32_t) start;
	
	if (debug_flag > 1) {
		kprintf("board_smp_start: cpu%d -> %x\n", cpu, start);
	}


	aw_h3_spinlock_lock((uint32_t) cpu);


	/* Set CPU boot address */
	out32(AW_H3_CPUCFG_BASE + CPUCFG_PRIVATE0_REG, startup_smp_start);


	/* Assert the CPU core in reset */
	out32(AW_H3_CPUCFG_BASE + CPUCFG_CPU_RST_CTRL_REG(cpu), 0);


	/* Assert the L1 cache in reset */
	reg = in32(AW_H3_CPUCFG_BASE + CPUCFG_GEN_CTRL_REG);
	out32(AW_H3_CPUCFG_BASE + CPUCFG_GEN_CTRL_REG, reg & ~BIT(cpu));


	/* Clear CPU power-off gating */
	reg = in32(AW_H3_PRCM_BASE + PRCM_CPU_PWROFF_REG);
	out32(AW_H3_PRCM_BASE + PRCM_CPU_PWROFF_REG, reg & ~BIT(cpu));
	mdelay(1);


	/* Deassert the CPU core reset */
	out32(AW_H3_CPUCFG_BASE + CPUCFG_CPU_RST_CTRL_REG(cpu), 3);


	aw_h3_spinlock_unlock((uint32_t) cpu);


	/* Send event to slave CPU to tell them to wake up */
	__asm__ __volatile__ ("sev" : : : "memory");

	return 1;
}

unsigned
board_smp_adjust_num(unsigned cpu)
{
	return cpu;
}
