/*
 * (c) 2023, SWD Embedded Systems Limited, http://www.kpda.ru
 */
 
/*
 * Allwinner H3 system reboot
 */

#include "callout.ah"
#include <arm/aw_h3.h>

/*
 * -----------------------------------------------------------------------
 * Routine to patch callout code
 *
 * On entry:
 *	r0 - physical address of syspage
 *	r1 - virtual  address of syspage
 *	r2 - offset from start of syspage to start of the callout routine
 *	r3 - offset from start of syspage to read/write data used by callout
 * -----------------------------------------------------------------------
 */
patch_reboot:
	stmdb	sp!,{r4,lr}
	add		r4, r0, r2					// address of callout routine

	/*
	 * Map Watchdog Timer registers
	 */
	mov		r0, #AW_H3_TIMER_SIZE
	ldr		r1, Lpaddr
	bl		callout_io_map

	/*
	 * Patch the callout routine
	 */
	CALLOUT_PATCH	r4, r0, r1, r2, ip
	ldmia	sp!,{r4,pc}

Lpaddr:	.word	AW_H3_TIMER_BASE

CALLOUT_START(reboot_awh3, 0, patch_reboot)
    /*
     * Get the Watchdog Timer base address (patched)
     */
    mov     ip,     #0x000000ff
    orr     ip, ip, #0x0000ff00
    orr     ip, ip, #0x00ff0000
    orr     ip, ip, #0xff000000

	/* disable interrupts */
	mrs		r0, cpsr
	orr		r0, r0, #0x80
	msr		cpsr, r0

	/* enable WDT */
	mov 	r0, #0x1
	str		r0, [ip, #WDOG0_CFG_REG]

	mov		r0, #0x0
	str		r0, [ip, #WDOG0_MODE_REG]

	ldr		r0, [ip, #WDOG0_MODE_REG]
	orr		r0, r0, #0x1
    str     r0, [ip, #WDOG0_MODE_REG]

0:	
    b		0b

CALLOUT_END(reboot_awh3)
