/*
 * (c) 2023, SWD Embedded Systems Limited, http://www.kpda.ru
 */

#include "startup.h"


static uint64_t start_cycle = 0;
static char *slog_buffer;
static unsigned slog_idx = 0;
static unsigned slog_size = 0;

#define TIMESTAMP_SIZE sizeof(uint64_t)
#define ALIGN_UP(n) (((n) + TIMESTAMP_SIZE) & ~(TIMESTAMP_SIZE - 1))

int slog_enabled = 0;

#define SERSLOG_BUFFER_OVERFLOW (UINT32_C(1) << 31)

static inline void write_index_slogbuffer(void)
{
    *(unsigned *)slog_buffer = slog_idx;
}

static inline void write_char_slogbuffer(char c)
{
	slog_buffer[slog_idx++] = (char)c;
}

static inline void write_timestamp_slogbuffer(void)
{
	slog_idx = ALIGN_UP(slog_idx);
	*(uint64_t *)&slog_buffer[slog_idx] = ClockCycles() - start_cycle;
	slog_idx += TIMESTAMP_SIZE;
}

static inline void set_overflow_slogbuffer(void)
{
	*(unsigned *)slog_buffer |= SERSLOG_BUFFER_OVERFLOW;
}

static inline int overflow_slogbuffer(void)
{
	if ((*(unsigned *)slog_buffer & SERSLOG_BUFFER_OVERFLOW) ||
		slog_idx + TIMESTAMP_SIZE >= slog_size) {
		return 1;
	}

	return 0;
}

static void init_slogbuffer(void)
{
	slog_idx = sizeof(unsigned);
	memset(slog_buffer, 0, slog_size);
	write_index_slogbuffer();
}

int init_slog(char *buffer, unsigned bufsize)
{
	start_cycle = ClockCycles();

	if (buffer == NULL || !bufsize) {
		return -1;
	}
	slog_buffer = buffer;
	slog_size = bufsize;

	init_slogbuffer();

	slog_enabled = 1;

	return 0;
}

void init_asinfo_slog(void)
{
	if (slog_enabled) {
		unsigned mem = as_find(AS_NULL_OFF, "memory", NULL);
		as_add((paddr_t)slog_buffer, (paddr_t)slog_buffer + slog_size - 1, AS_ATTR_ROM, "slog", mem);
	}
}

void *alloc_slog_buffer(size_t size, int fromtop)
{
	paddr_t addr = NULL_PADDR;
	if (fromtop) {
		addr = find_top_ram_aligned(size, 0, __PAGESIZE);
		if (addr == NULL_PADDR) {
			return NULL;
		}
	}
	addr = alloc_ram(addr, size, __PAGESIZE);
	if (addr == NULL_PADDR) {
		return NULL;
	}
	return startup_memory_map(size, addr, PROT_READ|PROT_WRITE);
}


void put_slog(int c)
{
	if (!overflow_slogbuffer()) {
		write_char_slogbuffer((char)c);
		if ((char)c == '\n') {
			write_timestamp_slogbuffer();
		}
		write_index_slogbuffer();
	} else {
		set_overflow_slogbuffer();
	}
}
