/*
 * (c) 2019, SWD Embedded Systems Limited, http://www.kpda.ru
 */

/*
 * $QNXLicenseC:
 * Licensed under the Apache License, Version 2.0 (the "License"). You 
 * may not reproduce, modify or distribute this software except in 
 * compliance with the License. You may obtain a copy of the License 
 * at: http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" basis, 
 * WITHOUT WARRANTIES OF ANY KIND, either express or implied.
 *
 * This file may contain contributions from others, either as 
 * contributors under the License or as licensors under other terms.  
 * Please review this entire file for other proprietary rights or license 
 * notices, as well as the QNX Development Suite License Guide at 
 * http://licensing.qnx.com/license-guide/ for other information.
 * $
 */


/*
 * uefi_ipl.h:	 IPL UEFI bootloader - definition of area 
 * for wrappers to call EFI 64-bit function form 32-bit startup
 *
 */

#ifndef _UEFI_IPL_H_INCLUDED
#define _UEFI_IPL_H_INCLUDED

#ifndef __PLATFORM_H_INCLUDED
#include <sys/platform.h>
#endif
#include <hw/uefi.h>

#if !defined(__X86_64__) 

typedef struct {
	_Uint64t	ipl_entry;         // 0x00 - entry point in ipl-uefi  (in 64-bit mode)
	_Uint64t	gdt_index;         // 0x08 - GDT index - base index in GDT used to jump to 32-bit and 64-bit
	_Uint64t	startup_retaddr;   // 0x10 - address to return to the startup (back to 32-bit mode)
	_Uint64t	efi_func_retval;   // 0x18 - field to store ret val of the efi function and return it in 32-bit startup
	_Uint64t	spare0;            // 0x20 - extra field for future use
	_Uint64t	spare1;            // 0x28 - extra field for future use
	_Uint64t	efi_func_addr;	   // 0x30 - addres of the 64-bit EFI service function
	_Uint64t	efi_func_nargs;	   // 0x38 - the 64-bit EFI service function number if arguments
	_Uint64t	args[32];          // 0x40 - area to store args  
} UEFI_IPL_WRAPPERS_AREA;


extern UEFI_IPL_WRAPPERS_AREA 	*uefi_ipl_wrappers_area;

EFI_STATUS uefi_wrapper_call64(void *func, int narg, ...);

#endif

#define UEFI_IPL_DATA_AREA_SIGNATURE 			0xFEDA9800EF100001ull

#define UEFI_IPL_SIGNATURE						0xEF100001
#define UEFI_IPL_FRAMEBUFFER_INFO_SIGNATURE		0xEF100002

typedef struct {
	_Uint32t	signature;
	_Uint32t	width;
	_Uint32t	height;
	_Uint32t	pixel_fmt;
	_Uint32t	pixels_per_line;
	_Uint32t	spare;
	_Uint64t	phys_base;
	_Uint64t	size;
} UEFI_IPL_FRAMEBUFFER_INFO;

typedef struct {
	_Uint64t 				signature;
	UEFI_IPL_FRAMEBUFFER_INFO   framebuffer;
}  __attribute__((packed)) UEFI_IPL_DATA_AREA;

extern UEFI_IPL_DATA_AREA 		*uefi_ipl_data_area;

#endif
